@@ -486,16 +486,21 @@ class EnvironmentContext(object):
            as a :class:`~sqlalchemy.schema.Table`,
            :class:`~sqlalchemy.schema.Column`,
            :class:`~sqlalchemy.schema.Index`
-           or :class:`~sqlalchemy.schema.UniqueConstraint` object
+           :class:`~sqlalchemy.schema.UniqueConstraint`,
+           or :class:`~sqlalchemy.schema.ForeignKeyConstraint` object
          * ``name``: the name of the object. This is typically available
            via ``object.name``.
          * ``type``: a string describing the type of object; currently
-           ``"table"``, ``"column"``, ``"index"`` or ``"unique_constraint"``.
+           ``"table"``, ``"column"``, ``"index"``, ``"unique_constraint"``,
+           or ``"foreign_key_constraint"``
 
            .. versionadded:: 0.7.0 Support for indexes and unique constraints
               within the
               :paramref:`~.EnvironmentContext.configure.include_object` hook.
 
+           .. versionadded:: 0.7.1 Support for foreign keys within the
+              :paramref:`~.EnvironmentContext.configure.include_object` hook.
+
          * ``reflected``: ``True`` if the given object was produced based on
            table reflection, ``False`` if it's from a local :class:`.MetaData`
            object.
@@ -689,7 +694,7 @@ class EnvironmentContext(object):
         opts['include_symbol'] = include_symbol
         opts['include_object'] = include_object
         opts['include_schemas'] = include_schemas
-        opts['render_as_batch'] = True #render_as_batch
+        opts['render_as_batch'] = render_as_batch
         opts['upgrade_token'] = upgrade_token
         opts['downgrade_token'] = downgrade_token
         opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
