@@ -1,22 +1,43 @@
 import sys
-from unittest import TestCase
+from alembic.testing import TestBase
+from alembic.testing import config
 
-from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
-    Numeric, DATETIME, INTEGER, \
-    TypeDecorator, Unicode, Enum,\
-    UniqueConstraint, Boolean, \
-    PrimaryKeyConstraint, Index, func, ForeignKeyConstraint,\
+from sqlalchemy import MetaData, Column, Table, Integer, String, \
+    Numeric, UniqueConstraint, Index, ForeignKeyConstraint,\
     ForeignKey
-from sqlalchemy.schema import AddConstraint
-from . import sqlite_db, eq_, db_for_dialect
+from alembic.testing import engines
+from alembic.testing import eq_
+from alembic.testing.env import staging_env
 
 py3k = sys.version_info >= (3, )
 
 from .test_autogenerate import AutogenFixtureTest
 
 
-class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestCase):
+class NoUqReflection(object):
+    __requires__ = ()
+
+    def setUp(self):
+        staging_env()
+        self.bind = eng = engines.testing_engine()
+
+        def unimpl(*arg, **kw):
+            raise NotImplementedError()
+        eng.dialect.get_unique_constraints = unimpl
+
+    @config.requirements.fail_before_sqla_083
+    def test_add_ix_on_table_create(self):
+        return super(NoUqReflection, self).test_add_ix_on_table_create()
+
+    @config.requirements.fail_before_sqla_080
+    def test_add_idx_non_col(self):
+        return super(NoUqReflection, self).test_add_idx_non_col()
+
+
+class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestBase):
     reports_unique_constraints = True
+    __requires__ = ('unique_constraint_reflection', )
+    __only_on__ = 'sqlite'
 
     def test_index_flag_becomes_named_unique_constraint(self):
         m1 = MetaData()
@@ -269,6 +290,42 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestCase):
         diffs = self._fixture(m1, m2)
         eq_(diffs, [])
 
+    def test_nothing_changed_implicit_composite_fk_index_named(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table("nothing_changed", m1,
+              Column('id', Integer, primary_key=True),
+              Column('other_id_1', Integer),
+              Column('other_id_2', Integer),
+              Column('foo', Integer),
+              ForeignKeyConstraint(
+                  ['other_id_1', 'other_id_2'], ['nc2.id1', 'nc2.id2'],
+                  name='fk_my_table_other_table'
+              ),
+              mysql_engine='InnoDB')
+        Table('nc2', m1,
+              Column('id1', Integer, primary_key=True),
+              Column('id2', Integer, primary_key=True),
+              mysql_engine='InnoDB')
+
+        Table("nothing_changed", m2,
+              Column('id', Integer, primary_key=True),
+              Column('other_id_1', Integer),
+              Column('other_id_2', Integer),
+              Column('foo', Integer),
+              ForeignKeyConstraint(
+                  ['other_id_1', 'other_id_2'], ['nc2.id1', 'nc2.id2'],
+                  name='fk_my_table_other_table'
+              ),
+              mysql_engine='InnoDB')
+        Table('nc2', m2,
+              Column('id1', Integer, primary_key=True),
+              Column('id2', Integer, primary_key=True),
+              mysql_engine='InnoDB')
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
     def test_new_idx_index_named_as_column(self):
         m1 = MetaData()
         m2 = MetaData()
@@ -435,10 +492,7 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestCase):
 
 class PGUniqueIndexTest(AutogenerateUniqueIndexTest):
     reports_unnamed_constraints = True
-
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('postgresql')
+    __only_on__ = "postgresql"
 
     def test_idx_added_schema(self):
         m1 = MetaData()
@@ -499,9 +553,29 @@ class PGUniqueIndexTest(AutogenerateUniqueIndexTest):
         eq_(diffs[0][0], "add_table")
         eq_(len(diffs), 1)
 
+    def test_uq_dropped(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table(
+            'add_uq', m1,
+            Column('id', Integer, primary_key=True),
+            Column('name', String),
+            UniqueConstraint('name', name='uq_name')
+        )
+        Table(
+            'add_uq', m2,
+            Column('id', Integer, primary_key=True),
+            Column('name', String),
+        )
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs[0][0], "remove_constraint")
+        eq_(diffs[0][1].name, "uq_name")
+        eq_(len(diffs), 1)
+
 
 class MySQLUniqueIndexTest(AutogenerateUniqueIndexTest):
     reports_unnamed_constraints = True
+    __only_on__ = 'mysql'
 
     def test_removed_idx_index_named_as_column(self):
         try:
@@ -512,22 +586,10 @@ class MySQLUniqueIndexTest(AutogenerateUniqueIndexTest):
         else:
             assert False, "unexpected success"
 
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('mysql')
 
-
-class NoUqReflectionIndexTest(AutogenerateUniqueIndexTest):
+class NoUqReflectionIndexTest(NoUqReflection, AutogenerateUniqueIndexTest):
     reports_unique_constraints = False
-
-    @classmethod
-    def _get_bind(cls):
-        eng = sqlite_db()
-
-        def unimpl(*arg, **kw):
-            raise NotImplementedError()
-        eng.dialect.get_unique_constraints = unimpl
-        return eng
+    __only_on__ = 'sqlite'
 
     def test_unique_not_reported(self):
         m1 = MetaData()
@@ -596,9 +658,11 @@ class NoUqReportsIndAsUqTest(NoUqReflectionIndexTest):
 
     """
 
+    __only_on__ = 'sqlite'
+
     @classmethod
     def _get_bind(cls):
-        eng = sqlite_db()
+        eng = config.db
 
         _get_unique_constraints = eng.dialect.get_unique_constraints
         _get_indexes = eng.dialect.get_indexes
@@ -618,3 +682,168 @@ class NoUqReportsIndAsUqTest(NoUqReflectionIndexTest):
         eng.dialect.get_unique_constraints = unimpl
         eng.dialect.get_indexes = get_indexes
         return eng
+
+
+class IncludeHooksTest(AutogenFixtureTest, TestBase):
+    __backend__ = True
+
+    def test_remove_connection_index(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        t1 = Table('t', m1, Column('x', Integer), Column('y', Integer))
+        Index('ix1', t1.c.x)
+        Index('ix2', t1.c.y)
+
+        Table('t', m2, Column('x', Integer), Column('y', Integer))
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            if type_ == 'unique_constraint':
+                return False
+            return not (
+                isinstance(object_, Index) and
+                type_ == 'index' and reflected and name == 'ix1')
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'remove_index')
+        eq_(diffs[0][1].name, 'ix2')
+        eq_(len(diffs), 1)
+
+    @config.requirements.unique_constraint_reflection
+    def test_remove_connection_uq(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            't', m1, Column('x', Integer), Column('y', Integer),
+            UniqueConstraint('x', name='uq1'),
+            UniqueConstraint('y', name='uq2'),
+        )
+
+        Table('t', m2, Column('x', Integer), Column('y', Integer))
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            if type_ == 'index':
+                return False
+            return not (
+                isinstance(object_, UniqueConstraint) and
+                type_ == 'unique_constraint' and reflected and name == 'uq1')
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'remove_constraint')
+        eq_(diffs[0][1].name, 'uq2')
+        eq_(len(diffs), 1)
+
+    def test_add_metadata_index(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('t', m1, Column('x', Integer))
+
+        t2 = Table('t', m2, Column('x', Integer))
+        Index('ix1', t2.c.x)
+        Index('ix2', t2.c.x)
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            return not (
+                isinstance(object_, Index) and
+                type_ == 'index' and not reflected and name == 'ix1')
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'add_index')
+        eq_(diffs[0][1].name, 'ix2')
+        eq_(len(diffs), 1)
+
+    @config.requirements.unique_constraint_reflection
+    def test_add_metadata_unique(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('t', m1, Column('x', Integer))
+
+        Table(
+            't', m2, Column('x', Integer),
+            UniqueConstraint('x', name='uq1'),
+            UniqueConstraint('x', name='uq2')
+        )
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            return not (
+                isinstance(object_, UniqueConstraint) and
+                type_ == 'unique_constraint' and
+                not reflected and name == 'uq1')
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'add_constraint')
+        eq_(diffs[0][1].name, 'uq2')
+        eq_(len(diffs), 1)
+
+    def test_change_index(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        t1 = Table(
+            't', m1, Column('x', Integer),
+            Column('y', Integer), Column('z', Integer))
+        Index('ix1', t1.c.x)
+        Index('ix2', t1.c.y)
+
+        t2 = Table(
+            't', m2, Column('x', Integer),
+            Column('y', Integer), Column('z', Integer))
+        Index('ix1', t2.c.x, t2.c.y)
+        Index('ix2', t2.c.x, t2.c.z)
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            return not (
+                isinstance(object_, Index) and
+                type_ == 'index' and not reflected and name == 'ix1'
+                and isinstance(compare_to, Index))
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'remove_index')
+        eq_(diffs[0][1].name, 'ix2')
+        eq_(diffs[1][0], 'add_index')
+        eq_(diffs[1][1].name, 'ix2')
+        eq_(len(diffs), 2)
+
+    @config.requirements.unique_constraint_reflection
+    def test_change_unique(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            't', m1, Column('x', Integer),
+            Column('y', Integer), Column('z', Integer),
+            UniqueConstraint('x', name='uq1'),
+            UniqueConstraint('y', name='uq2')
+        )
+
+        Table(
+            't', m2, Column('x', Integer), Column('y', Integer),
+            Column('z', Integer),
+            UniqueConstraint('x', 'z', name='uq1'),
+            UniqueConstraint('y', 'z', name='uq2')
+        )
+
+        def include_object(object_, name, type_, reflected, compare_to):
+            if type_ == 'index':
+                return False
+            return not (
+                isinstance(object_, UniqueConstraint) and
+                type_ == 'unique_constraint' and
+                not reflected and name == 'uq1'
+                and isinstance(compare_to, UniqueConstraint))
+
+        diffs = self._fixture(m1, m2, object_filters=[include_object])
+
+        eq_(diffs[0][0], 'remove_constraint')
+        eq_(diffs[0][1].name, 'uq2')
+        eq_(diffs[1][0], 'add_constraint')
+        eq_(diffs[1][1].name, 'uq2')
+        eq_(len(diffs), 2)
