@@ -10,7 +10,9 @@ from alembic.batch import ApplyBatchImpl
 from alembic.migration import MigrationContext
 
 from sqlalchemy import Integer, Table, Column, String, MetaData, ForeignKey, \
-    UniqueConstraint, ForeignKeyConstraint, Index
+    UniqueConstraint, ForeignKeyConstraint, Index, Boolean, CheckConstraint, \
+    Enum
+from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.sql import column
 from sqlalchemy.schema import CreateTable, CreateIndex
 
@@ -53,6 +55,30 @@ class BatchApplyTest(TestBase):
         )
         return ApplyBatchImpl(t, table_args, table_kwargs)
 
+    def _literal_ck_fixture(
+            self, copy_from=None, table_args=(), table_kwargs={}):
+        m = MetaData()
+        if copy_from is not None:
+            t = copy_from
+        else:
+            t = Table(
+                'tname', m,
+                Column('id', Integer, primary_key=True),
+                Column('email', String()),
+                CheckConstraint("email LIKE '%@%'")
+            )
+        return ApplyBatchImpl(t, table_args, table_kwargs)
+
+    def _sql_ck_fixture(self, table_args=(), table_kwargs={}):
+        m = MetaData()
+        t = Table(
+            'tname', m,
+            Column('id', Integer, primary_key=True),
+            Column('email', String())
+        )
+        t.append_constraint(CheckConstraint(t.c.email.like('%@%')))
+        return ApplyBatchImpl(t, table_args, table_kwargs)
+
     def _fk_fixture(self, table_args=(), table_kwargs={}):
         m = MetaData()
         t = Table(
@@ -83,6 +109,33 @@ class BatchApplyTest(TestBase):
         )
         return ApplyBatchImpl(t, table_args, table_kwargs)
 
+    def _boolean_fixture(self, table_args=(), table_kwargs={}):
+        m = MetaData()
+        t = Table(
+            'tname', m,
+            Column('id', Integer, primary_key=True),
+            Column('flag', Boolean)
+        )
+        return ApplyBatchImpl(t, table_args, table_kwargs)
+
+    def _boolean_no_ck_fixture(self, table_args=(), table_kwargs={}):
+        m = MetaData()
+        t = Table(
+            'tname', m,
+            Column('id', Integer, primary_key=True),
+            Column('flag', Boolean(create_constraint=False))
+        )
+        return ApplyBatchImpl(t, table_args, table_kwargs)
+
+    def _enum_fixture(self, table_args=(), table_kwargs={}):
+        m = MetaData()
+        t = Table(
+            'tname', m,
+            Column('id', Integer, primary_key=True),
+            Column('thing', Enum('a', 'b', 'c'))
+        )
+        return ApplyBatchImpl(t, table_args, table_kwargs)
+
     def _assert_impl(self, impl, colnames=None,
                      ddl_contains=None, ddl_not_contains=None,
                      dialect='default'):
@@ -129,8 +182,8 @@ class BatchApplyTest(TestBase):
                     "CAST(tname.%s AS %s) AS anon_1" % (
                         name, impl.new_table.c[name].type)
                     if (
-                        impl.new_table.c[name].type
-                        is not impl.table.c[name].type)
+                        impl.new_table.c[name].type._type_affinity
+                        is not impl.table.c[name].type._type_affinity)
                     else "tname.%s" % name
                     for name in colnames if name in impl.table.c
                 )
@@ -153,6 +206,92 @@ class BatchApplyTest(TestBase):
         new_table = self._assert_impl(impl)
         eq_(new_table.c.x.name, 'q')
 
+    def test_rename_col_boolean(self):
+        impl = self._boolean_fixture()
+        impl.alter_column('tname', 'flag', name='bflag')
+        new_table = self._assert_impl(
+            impl, ddl_contains="CHECK (bflag IN (0, 1)",
+            colnames=["id", "flag"])
+        eq_(new_table.c.flag.name, 'bflag')
+        eq_(
+            len([
+                const for const
+                in new_table.constraints
+                if isinstance(const, CheckConstraint)]),
+            1)
+
+    def test_rename_col_boolean_no_ck(self):
+        impl = self._boolean_no_ck_fixture()
+        impl.alter_column('tname', 'flag', name='bflag')
+        new_table = self._assert_impl(
+            impl, ddl_not_contains="CHECK",
+            colnames=["id", "flag"])
+        eq_(new_table.c.flag.name, 'bflag')
+        eq_(
+            len([
+                const for const
+                in new_table.constraints
+                if isinstance(const, CheckConstraint)]),
+            0)
+
+    def test_rename_col_enum(self):
+        impl = self._enum_fixture()
+        impl.alter_column('tname', 'thing', name='thang')
+        new_table = self._assert_impl(
+            impl, ddl_contains="CHECK (thang IN ('a', 'b', 'c')",
+            colnames=["id", "thing"])
+        eq_(new_table.c.thing.name, 'thang')
+        eq_(
+            len([
+                const for const
+                in new_table.constraints
+                if isinstance(const, CheckConstraint)]),
+            1)
+
+    def test_rename_col_literal_ck(self):
+        impl = self._literal_ck_fixture()
+        impl.alter_column('tname', 'email', name='emol')
+        new_table = self._assert_impl(
+            # note this is wrong, we don't dig into the SQL
+            impl, ddl_contains="CHECK (email LIKE '%@%')",
+            colnames=["id", "email"])
+        eq_(
+            len([c for c in new_table.constraints
+                if isinstance(c, CheckConstraint)]), 1)
+
+        eq_(new_table.c.email.name, 'emol')
+
+    def test_rename_col_literal_ck_workaround(self):
+        impl = self._literal_ck_fixture(
+            copy_from=Table(
+                'tname', MetaData(),
+                Column('id', Integer, primary_key=True),
+                Column('email', String),
+            ),
+            table_args=[CheckConstraint("emol LIKE '%@%'")])
+
+        impl.alter_column('tname', 'email', name='emol')
+        new_table = self._assert_impl(
+            impl, ddl_contains="CHECK (emol LIKE '%@%')",
+            colnames=["id", "email"])
+        eq_(
+            len([c for c in new_table.constraints
+                if isinstance(c, CheckConstraint)]), 1)
+        eq_(new_table.c.email.name, 'emol')
+
+    def test_rename_col_sql_ck(self):
+        impl = self._sql_ck_fixture()
+
+        impl.alter_column('tname', 'email', name='emol')
+        new_table = self._assert_impl(
+            impl, ddl_contains="CHECK (emol LIKE '%@%')",
+            colnames=["id", "email"])
+        eq_(
+            len([c for c in new_table.constraints
+                if isinstance(c, CheckConstraint)]), 1)
+
+        eq_(new_table.c.email.name, 'emol')
+
     def test_add_col(self):
         impl = self._simple_fixture()
         col = Column('g', Integer)
@@ -302,10 +441,11 @@ class BatchAPITest(TestBase):
     __requires__ = ('sqlalchemy_08', )
 
     @contextmanager
-    def _fixture(self):
+    def _fixture(self, schema=None):
         migration_context = mock.Mock(opts={})
         op = Operations(migration_context)
-        batch = op.batch_alter_table('tname', recreate='never').__enter__()
+        batch = op.batch_alter_table(
+            'tname', recreate='never', schema=schema).__enter__()
 
         with mock.patch("alembic.operations.sa_schema") as mock_schema:
             yield batch
@@ -344,8 +484,60 @@ class BatchAPITest(TestBase):
                 mock.call(
                     ['x'], ['user.y'],
                     onupdate=None, ondelete=None, name='myfk',
-                    initially=None, deferrable=None, match=None,
-                    schema=None)
+                    initially=None, deferrable=None, match=None)
+            ]
+        )
+        eq_(
+            self.mock_schema.Table.mock_calls,
+            [
+                mock.call(
+                    'user', self.mock_schema.MetaData(),
+                    self.mock_schema.Column(),
+                    schema=None
+                ),
+                mock.call(
+                    'tname', self.mock_schema.MetaData(),
+                    self.mock_schema.Column(),
+                    schema=None
+                ),
+                mock.call().append_constraint(
+                    self.mock_schema.ForeignKeyConstraint())
+            ]
+        )
+        eq_(
+            batch.impl.operations.impl.mock_calls,
+            [mock.call.add_constraint(
+                self.mock_schema.ForeignKeyConstraint())]
+        )
+
+    def test_create_fk_schema(self):
+        with self._fixture(schema='foo') as batch:
+            batch.create_foreign_key('myfk', 'user', ['x'], ['y'])
+
+        eq_(
+            self.mock_schema.ForeignKeyConstraint.mock_calls,
+            [
+                mock.call(
+                    ['x'], ['user.y'],
+                    onupdate=None, ondelete=None, name='myfk',
+                    initially=None, deferrable=None, match=None)
+            ]
+        )
+        eq_(
+            self.mock_schema.Table.mock_calls,
+            [
+                mock.call(
+                    'user', self.mock_schema.MetaData(),
+                    self.mock_schema.Column(),
+                    schema=None
+                ),
+                mock.call(
+                    'tname', self.mock_schema.MetaData(),
+                    self.mock_schema.Column(),
+                    schema='foo'
+                ),
+                mock.call().append_constraint(
+                    self.mock_schema.ForeignKeyConstraint())
             ]
         )
         eq_(
@@ -428,9 +620,10 @@ class BatchRoundTripTest(TestBase):
         self.metadata.drop_all(self.conn)
         self.conn.close()
 
-    def _assert_data(self, data):
+    def _assert_data(self, data, tablename='foo'):
         eq_(
-            [dict(row) for row in self.conn.execute("select * from foo")],
+            [dict(row) for row
+             in self.conn.execute("select * from %s" % tablename)],
             data
         )
 
@@ -482,6 +675,31 @@ class BatchRoundTripTest(TestBase):
             {"id": 5, "x": 9}
         ])
 
+    @config.requirements.sqlalchemy_094
+    @config.requirements.unnamed_constraints
+    def test_drop_foreign_key(self):
+        bar = Table(
+            'bar', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('foo_id', Integer, ForeignKey('foo.id')),
+            mysql_engine='InnoDB'
+        )
+        bar.create(self.conn)
+        self.conn.execute(bar.insert(), {'id': 1, 'foo_id': 3})
+
+        naming_convention = {
+            "fk":
+            "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
+        }
+        with self.op.batch_alter_table(
+                "bar", naming_convention=naming_convention) as batch_op:
+            batch_op.drop_constraint(
+                "fk_bar_foo_id_foo", type_="foreignkey")
+        eq_(
+            Inspector.from_engine(self.conn).get_foreign_keys('bar'),
+            []
+        )
+
     def test_drop_column_fk_recreate(self):
         with self.op.batch_alter_table("foo", recreate='always') as batch_op:
             batch_op.drop_column('data')
@@ -506,6 +724,54 @@ class BatchRoundTripTest(TestBase):
             {"id": 5, "data": "d5", "y": 9}
         ])
 
+    def test_rename_column_boolean(self):
+        bar = Table(
+            'bar', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('flag', Boolean()),
+            mysql_engine='InnoDB'
+        )
+        bar.create(self.conn)
+        self.conn.execute(bar.insert(), {'id': 1, 'flag': True})
+        self.conn.execute(bar.insert(), {'id': 2, 'flag': False})
+
+        with self.op.batch_alter_table(
+            "bar"
+        ) as batch_op:
+            batch_op.alter_column(
+                'flag', new_column_name='bflag', existing_type=Boolean)
+
+        self._assert_data([
+            {"id": 1, 'bflag': True},
+            {"id": 2, 'bflag': False},
+        ], 'bar')
+
+    @config.requirements.non_native_boolean
+    def test_rename_column_non_native_boolean_no_ck(self):
+        bar = Table(
+            'bar', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('flag', Boolean(create_constraint=False)),
+            mysql_engine='InnoDB'
+        )
+        bar.create(self.conn)
+        self.conn.execute(bar.insert(), {'id': 1, 'flag': True})
+        self.conn.execute(bar.insert(), {'id': 2, 'flag': False})
+        self.conn.execute(bar.insert(), {'id': 3, 'flag': 5})
+
+        with self.op.batch_alter_table(
+            "bar",
+            reflect_args=[Column('flag', Boolean(create_constraint=False))]
+        ) as batch_op:
+            batch_op.alter_column(
+                'flag', new_column_name='bflag', existing_type=Boolean)
+
+        self._assert_data([
+            {"id": 1, 'bflag': True},
+            {"id": 2, 'bflag': False},
+            {'id': 3, 'bflag': 5}
+        ], 'bar')
+
     def test_drop_column_pk(self):
         with self.op.batch_alter_table("foo") as batch_op:
             batch_op.drop_column('id')
